<?php
namespace Quivers\Integration\Controller\Index;
use Stripe\Stripe;
use Quivers\Integration\LogTracking;

class Index extends \Magento\Framework\App\Action\Action
{
    protected $_pageFactory;
 
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\View\Result\PageFactory $pageFactory)
    {
        $this->_pageFactory = $pageFactory;
        $this->_checkoutSession = $checkoutSession;
        return parent::__construct($context);
        
    }
    
    const path = 'ecommerce';
    public function execute()
    {   
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $post = $this->getRequest()->getParam('input_val');
        $urlInterface = $objectManager->get('Magento\Framework\UrlInterface');
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $log = new LogTracking($scopeConfig);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
        $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $description=$quote->getBillingAddress()->getEmail();
        $phone = $quote->getShippingAddress()->getTelephone();
       
        $baseUrl=$storeManager->getStore()->getBaseUrl();
            $log->debug_mode_on($debug_mod,
                'INDEX.php execute- '.$description . "-".$phone,
                null, null ,[
                    "basepath"=>$baseUrl.self::path,
                    "currenturl"=>$urlInterface->getCurrentUrl()
                ]);

     
       //base url path=base url + /ecommerce;
        if($baseUrl.self::path==$urlInterface->getCurrentUrl()){
            return  $this->_pageFactory->create();
        }
     
        $client_secret=$this->createPaymentIntent();
        $response = $this->resultFactory
            ->create(\Magento\Framework\Controller\ResultFactory::TYPE_JSON)
            ->setData([
                'status'  => "success",
                'client_secret' => isset($client_secret)?$client_secret:''
            ]);

        return $response;
    }

    public function createPaymentIntent()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); 
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
        $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $log = new LogTracking($scopeConfig);
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        try
        {   
            $secret_key = $scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
            Stripe::setApiKey($secret_key);
            #get stripe customer id
     
            $customer=$customer_session->getCustomer();
            $customer_id=$customer->getQuiversStripeCustomerId();
            $payment=$quote->getPayment();

            #get total amount 
            $grandTotal=$payment->getAdditionalInformation('QuiversTotal');
            if(!$grandTotal){
                $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
                $newGrandTotal = $cart->getQuote()->getGrandTotal();
                
                if($newGrandTotal){
                    $grandTotal = $newGrandTotal;
                }
            }

            #store credit code
            $cartTotalRepository = $objectManager->get('\Magento\Quote\Api\CartTotalRepositoryInterface');
            $totals = $cartTotalRepository->get($quote->getId());
            $storeCredit = 0.0;
            $totalSegmentArray = [];
            $totalSegments= [];
            foreach($totals->getTotalSegments() as $totalSegment){
                $totalSegmentArray = $totalSegment->toArray();
                if ($totalSegmentArray["code"] == "customerbalance"){
                    $storeCredit = $totalSegmentArray["value"];
                }
                array_push($totalSegments, $totalSegmentArray);
            }
            if (is_numeric($storeCredit)){
                if($storeCredit<0){
                    $grandTotal = $grandTotal + $storeCredit;
                }
                elseif($storeCredit>0){
                    $grandTotal = $grandTotal - $storeCredit;
                }
            }
            else{
                str_replace("-","",$storeCredit);
                $storeCredit = floatval($storeCredit);
                $grandTotal = $grandTotal - $storeCredit;
            }

            #get currency code
            $cart = $objectManager->get('\Magento\Checkout\Model\Cart');
            $currency= $cart->getQuote()->getQuoteCurrencyCode();
               

   

            #create charge amount
            $payment->setAdditionalInformation('QuiversStripeChargeAmount', $grandTotal);
            $chargeAmount=$this->formatCurrencyAmount($grandTotal, $currency);
            

            #get  statement_descriptor_suffix
            $statement_descriptor_suffix = $payment->getAdditionalInformation('payment_descriptor');
            $payment_token = $payment->getAdditionalInformation('cc_stripejs_token');
            if(strlen($statement_descriptor_suffix) > 10)
            {
                $statement_descriptor_suffix = substr($statement_descriptor_suffix, 0, 10);
            }

            //debug mode
            $LogData = [
                "GrandTotal 1" => $customer_session->getTotalAmount(),
                "GrandTotal 2" => $payment->getAdditionalInformation('QuiversTotal'),
                "Grand Total 3" => $cart->getQuote()->getGrandTotal(),
                "Final grandtotal"=>$grandTotal,
                "storeCredit" => $storeCredit,
                "totalSegments"=>$totalSegments,
                "currency"=> $currency,
                "statement_descriptor_suffix" => $statement_descriptor_suffix
            ];
            
            #creating desciption
            $description=$quote->getBillingAddress()->getEmail();
            if(!$description){
                throw new \Magento\Framework\Validator\Exception(__(' No Email id provided'));
            }

            $log->debug_mode_on($debug_mod,
                'INDEX.php total check- '.$description,
                null, null ,$LogData);
            
            // check and create a customer before create intent
            $customer = false;
            $customersList = \Stripe\Customer::all(array(
                "limit" => 1,
                "email" => $description,
            ));
         
            // create a new user if it doesn't exists.
            if(count($customersList["data"])>0) {
                $customer = $customersList["data"][0];
            }
            //debug mode

                $log->debug_mode_on($debug_mod,
                    'authorize customer check index.php- '.$description,
                    null, null ,["customer"=>$customer,"payment_token"=>$payment_token]);

           
            if(!$customer){
                $customer = \Stripe\Customer::create(array(
                'source' => $payment_token,
                'email' => $description,
                ));
                    // //debug mode


                    $log->debug_mode_on($debug_mod,
                        'authorize inside customer create index.php- '.$description,
                        null, null ,$customer);

            }
            if(!$customer){
                throw new \Magento\Framework\Validator\Exception(__(' Unable to Fetch  Customer Details From Stripe'));
            }
            $stripeCustomeID = isset($customer->id)?$customer->id:null;
            $customerData = $customer_session->getCustomer();
            $customerData->setAttributeSetId(\Magento\Customer\Api\CustomerMetadataInterface::ATTRIBUTE_SET_ID_CUSTOMER);
            $customerData->setQuiversStripeCustomerId($stripeCustomeID);
                //
            #creating payment intent payload
            $payload=array(
                    "amount" => $chargeAmount, 
                    "currency" => $currency,
                    "confirmation_method" => 'automatic',
                    "capture_method" => "manual",
                    "customer"=>$stripeCustomeID,
                    // "source"=>$payment_token,
                    "setup_future_usage"=>'off_session',
                    "description"=>$description,
                    "statement_descriptor_suffix"=>$statement_descriptor_suffix,

                );
         
                
            #get destination
            if($payment->getAdditionalInformation('destination') === true){
                $merchantId = $payment->getAdditionalInformation('merchantId');
                if($merchantId != '0'){
                    $transferData = array(
                        "destination" => $merchantId,
                        "amount" => $chargeAmount
                    );
                    $onBehalfOf = $merchantId;
                    $payload['transfer_data'] = $transferData;
                    $payload['on_behalf_of'] = $onBehalfOf;
                }
            }

            #creating payment intent
            $intent = \Stripe\PaymentIntent::create($payload);
            // //debug mode

                $log->debug_mode_on($debug_mod,
                    'createPaymentIntent inside payload and intent response index.php- '.$description,
                    $payload, $intent ,null);

            // #set paymentintent id to session
            $customer_session->setPaymentIntentId($intent->id);
            return $intent->client_secret;
        }
        catch(\Exception $e)
        {
                $log->debug_mode_on($debug_mod,'catch index.php- '.$quote->getBillingAddress()->getEmail(),
                    null, null ,$e->getMessage());
               return  '';
        }
    }

    public function formatCurrencyAmount($amount, $currency)
    {

        // Stripe takes amounts as the smallest currency unit. ...
        if (empty($amount)) {
            return null;
        }
        $multiple = 100;
        if (in_array(
            $currency,
            array('BIF', 'DJF', 'JPY', 'KRW', 'PYG', 'VND', 'XAF', 'XPF', 'CLP', 'GNF', 'KMF', 'MGA', 'RWF', 'VUV', 'XOF'),
            true
        )) {
            $multiple = 1;
        }

        return sprintf('%01.0f', (float)$amount * $multiple);
    }
}
