<?php

namespace Quivers\Integration\Controller\Onepage;

use Magento\Framework\App\Action\HttpGetActionInterface as HttpGetActionInterface;
use Quivers\Integration\LogTracking;

/**
 * Onepage checkout success controller class
 */
class Success extends \Magento\Checkout\Controller\Onepage implements HttpGetActionInterface
{
    /**
     * Order success action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {   
        $session = $this->getOnepage()->getCheckout();
        $session->clearQuote();
        $startTime = microtime(true);
        $orderId=$session->getLastOrderId();
        $lastOrderId=$session->getLastRealOrder();
        //@todo: Refactor it to match CQRS
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $order = $objectManager->create('Magento\Sales\Api\Data\OrderInterface')->load($orderId);

        $method = $order->getPayment()->getMethod();

        if( $method == 'quivers_integration'){
            $customerSession = $objectManager->get('Magento\Customer\Model\Session');
            $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
            $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
                  
            $payment_intentId=$customerSession->getMyValue();
           
            $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
            $log = new LogTracking($scopeConfig);
            $secret_key = $scopeConfig->getValue("payment/quivers_integration/secret_key", $storeScope);
            $stripe= new \Stripe\StripeClient($secret_key);
            
            $payment_Intent=$stripe->paymentIntents->retrieve(
                $payment_intentId,
                []
            );  
            if($debug_mod){
                $log->debug_mode_on($debug_mod,
                    'Order Created,Payment Intent created, need to confirm 3D payment for order ID- '.$orderId,
                    null, null ,$payment_Intent);
            }        
            if ($payment_Intent["status"]== "requires_action"|| $payment_Intent["status"]== "requires_source_action"){    
                
                $payment_Intent=$stripe->paymentIntents->update(
                    $payment_intentId,
                    ['metadata' => ['order_id' => $orderId]]
                  );
     
                $newURL=$payment_Intent->next_action->redirect_to_url->url;
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'Redirecting for  3D payment confirmation for order ID- '.$orderId,
                        null, null ,$payment_Intent);
                }
                $redirect = $this->resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);
                $redirect->setUrl($newURL);
    
                return $redirect;  
            }
            else if (isset($payment_Intent["status"]) && $payment_Intent["status"]== "requires_capture"){

                if($orderId!=$payment_Intent->charges->data[0]->metadata->order_id){
                    $orderId=$payment_Intent->charges->data[0]->metadata->order_id;
                    $order = $objectManager->create('Magento\Sales\Api\Data\OrderInterface')->load($orderId);
                }
                
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'Redirecting to success page after 3D confirmation. Payment Intent requires capture. Customer has sucessfully confirmed the payment for  - '.$orderId,
                        null, null ,$payment_Intent);
                 }
    
                $payment_Intent = $stripe->paymentIntents->capture(
                    $payment_intentId,
                    []
                 );
                 if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'Redirecting to success page after 3D confirmation. Payment Intent captured succesfully for Order ID - '.$orderId,
                        null, null ,$payment_Intent);
                 }

               $order_data = [ 
                "Email"=>$order->getCustomerEmail() 
                ];

               $payment = $order->getPayment();
               $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING, true)->save();
               $order->setStatus(\Magento\Sales\Model\Order::STATE_PROCESSING, true)->save();
               $payment->setLastTransId($payment_Intent->charges->data[0]->id);
               $payment->setTransactionId($payment_Intent->charges->data[0]->id);
        
               $messageD = __('The Amount is authorized. ');
               $trans = $objectManager->create("\Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface");
               $transaction = $trans->setPayment($payment)
                ->setOrder($order)
                ->setTransactionId($payment_Intent->charges->data[0]->id)
                ->setFailSafe(true)
                ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_AUTH);
        
               $payment->addTransactionCommentsToOrder(
                    $transaction,
                    $messageD
                );
               $payment->setParentTransactionId(null);
               $payment->save();
               $order->save();
               $response = ["Response"=>$payment_Intent->charges->data[0], "Amount"=>$payment_Intent->charges->data[0]->amount,'payment_method'=>$payment_Intent->charges->data[0]->payment_method];
               $log->final_payment_intent("stripe", date('r', $startTime), $payment_Intent);
               $log->payment_request($order_data, "stripe", date('r', $startTime), $response, $payment_Intent->status);
               $customerSession->unsetPaymentIntentId();
               $customerSession->unsetTotalAmount();

               $resultPage = $this->resultPageFactory->create();
               $this->_eventManager->dispatch(
                    'checkout_onepage_controller_success_action',
                    [
                        'order_ids' => [$orderId],
                        'order' => $lastOrderId
                    ]
                );
                return $resultPage;

            }
            else if($payment_Intent["status"]== "requires_source"){
                if(isset($payment_Intent->metadata->order_id) ){
                    if($orderId!=$payment_Intent->metadata->order_id){
                        $orderId=$payment_Intent->metadata->order_id;
                        $order = $objectManager->create('Magento\Sales\Api\Data\OrderInterface')->load($orderId);
                    }
                }
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'Payment rejected by customer while on 3D confirmation for  - '.$orderId,
                        null, null ,$payment_Intent);
                }
                $order->setState(\Magento\Sales\Model\Order::STATE_CANCELED, true)->save();
                $order->setStatus(\Magento\Sales\Model\Order::STATE_CANCELED, true)->save();
                $order->addStatusToHistory($order->getStatus(), 'Payment Declined while 3d confirmation');
                $order->save();
                return $this->resultRedirectFactory->create()->setPath('checkout/cart');

            }
            else if (isset($payment_Intent["status"]) && $payment_Intent["status"]== "succeeded"){
                if($debug_mod){
                    $log->debug_mode_on($debug_mod,
                        'Payment sucessfully completed for Order ID - '.$orderId,
                        null, null ,$payment_Intent);
                 }
                $resultPage = $this->resultPageFactory->create();
                 $this->_eventManager->dispatch(
                     'checkout_onepage_controller_success_action',
                     [
                         'order_ids' => [$orderId],
                         'order' => $lastOrderId
                     ]
                );
                 return $resultPage;
            }

        }


        $resultPage = $this->resultPageFactory->create();
        $this->_eventManager->dispatch(
            'checkout_onepage_controller_success_action',
            [
                'order_ids' => [$orderId],
                'order' => $lastOrderId
            ]
        );
        return $resultPage;   
    }
}
