<?php

/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Quivers\Integration\Controller\Sales\Adminhtml\Order;
use Quivers\Integration\LogTracking;
class ValidateOrderAction
{

    /**
     * Validate if unhold action can be performed or not.
     * @return 
     */
    public static function validateOrderAction(
        $orderId,
        $quiversSandbox,

        $quiversAPIKey,
        $merchantREFId,
        $curlClient
    ) {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $log = new LogTracking($scopeConfig);
        $orderUpdateValidateUrl = 'https://middleware.quiversdemo.com/profile/order/status';
        if ($quiversSandbox == 0) {
            $orderUpdateValidateUrl = "https://middleware.quivers.com/profile/order/status";
        }

        $payload = array(
            'client_type' => 'Magento2',
            'profile_id' => $merchantREFId,
            'order_id' => $orderId,
        );
        try{
            $curlClient->addHeader("Content-Type", "application/json");
            $curlClient->addHeader("Authorization", "apikey " . $quiversAPIKey);
    
            # Setup request to send json via POST.
            $curlClient->post($orderUpdateValidateUrl, json_encode($payload));
    
            $response = $curlClient->getBody();
            if ($curlClient->getStatus() != 200) {
                $errorMessage = json_decode($response)->message;
                
                $log->debug_mode_on($debug_mod,
                'Curl client status is not OK ->'.$orderId,
                null,  null,["URL"=>$orderUpdateValidateUrl,"Get Status"=>$curlClient->getStatus(),"Error Message"=>$errorMessage],);
                return false;
            }
            else{
                $responseDecoded = json_decode($response, true);
                if ($debug_mod) {
                    $logData = [
                        "Url" => $orderUpdateValidateUrl,
                        "Payload" => $payload,
                        "Status" => $curlClient->getStatus(),
                    ];
                    $log->debug_mode_on(
                        $debug_mod,
                        'Validate Order Action for OrderId - ' . $orderId,
                        null,
                        $response,
                        $logData,
                    );
                }
                foreach ($responseDecoded as $key => $item) {
                    if ($item['Value'] == 60 && $item['performed_by'] == 'Retailer') {
                        return false;
                    }
                }
        
                return true;
            }
        }
        catch(\Exception $e)
        {
            $log->debug_mode_on($debug_mod,
			'Catch  "ValidateorderAction.php " got an error with status '.$curlClient->getStatus(),
			null,  null,$e->getMessage());

        }
    }
}
