<?php

namespace Quivers\Integration\Observer;

use Quivers\Integration\Block\Adminhtml\Form\Button;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;
use Quivers\Integration\LogTracking;

class ConfigObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
    protected $curlClient;
    protected $scopeConfig;
    protected $configWriter;
    protected $configValueFactory;
    protected $_responseFactory;
    public $layout;
    public $request;
    public $currentStoreId;
    public $_backendUrl;
    protected $_productCollectionFactory;

    /**
     * @param Logger $logger
     */
    
    const XML_MARKETPLACE_ID = 'quivers/parameters/marketplaceId';
    const XML_PRODUCTGROUP_ID = 'quivers/parameters/ProductGroupId';
    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_MAGENTO_ACCESS_TOKEN = 'quivers/parameters/magento_access_token';
    const XML_MAGENTO_BASE_URL = 'quivers/parameters/magento_base_url';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EUR_MARKETPLACE_ID = 'quivers_tax/marketplaces/eurMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';
    const XML_QUIVERS_STATUS_LABEL = 'quivers/parameters/quivers_status_label';
    const XML_QUIVERS_UUID = 'quivers/parameters/quivers_uuid';
    const XML_ENABLED = 'quivers/parameters/active';
    const XML_QUIVERS_UPC = 'quivers/parameters/quivers_custom_upc';
    const XML_QUIVERS_PREORDER = 'quivers/parameters/quivers_custom_Pre_Order';
    const XML_QUIVERS_PENDING_RETAILER_FULFILLMENT_STATUS = 'quivers/parameters/pending_retailer_fulfillment_status';


    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';

    public function __construct(
        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
        \Magento\Framework\App\Config\ValueFactory $configValueFactory,
        \Magento\Framework\View\LayoutInterface $layout,
        \Magento\Backend\Model\Session $backendSession,
        \Magento\Catalog\Model\Session $catalogSession,
        \Magento\Backend\Model\UrlInterface $backendUrl,
        \Magento\Framework\App\ResponseFactory $responseFactory,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory
    ) {
        $this->curlClient = $curl;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->configWriter = $configWriter;
        $this->configValueFactory = $configValueFactory;
        $this->layout = $layout;
        $this->backendSession = $backendSession;
        $this->_catalogSession = $catalogSession;
        $this->_backendUrl = $backendUrl;
        $this->_responseFactory = $responseFactory;
        $this->request = $request;
        $this->storeManager = $storeManager;
        $this->_productCollectionFactory = $productCollectionFactory;
    }

    public function execute(EventObserver $observer)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
        $blockObj = $this->layout->createBlock('Quivers\Integration\Block\Adminhtml\Form\Button');
        
        $websiteScope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITE;
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $websiteId = (int) $this->request->getParam('website',0);
        $storeId = (int) $this->request->getParam('store',0);
        $scopeConfig = \Magento\Framework\App\ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
        $debug_mod = $scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope);
        $checkout_session=$objectManager->get('\Magento\Checkout\Model\Session');
        $quote = $checkout_session->getQuote();
        $log = new LogTracking($this->scopeConfig);
        if($websiteId){
            $marketPlaceId = $this->scopeConfig->getValue(self::XML_MARKETPLACE_ID, $websiteScope,$websiteId);
            $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $websiteScope,$websiteId);
            $eurMarketPlaceId = $this->scopeConfig->getValue(self::XML_EUR_MARKETPLACE_ID, $websiteScope,$websiteId);
            $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $websiteScope,$websiteId);
            $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $websiteScope,$websiteId);
            $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $websiteScope,$websiteId);
            $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $websiteScope,$websiteId);
            $productGroupId = $this->scopeConfig->getValue(self::XML_PRODUCTGROUP_ID, $websiteScope,$websiteId);
            $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $websiteScope, $websiteId);
            $magentoBaseUrl = $this->scopeConfig->getValue(self::XML_MAGENTO_BASE_URL, $websiteScope,$websiteId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $websiteScope,$websiteId);
            $quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $websiteScope,$websiteId);
            $magentoAcessToken = $this->scopeConfig->getValue(self::XML_MAGENTO_ACCESS_TOKEN, $websiteScope,$websiteId);
            $active = $this->scopeConfig->getValue(self::XML_ENABLED, $websiteScope, $storeId);
            $quivers_upc = $this->scopeConfig->getValue(self::XML_QUIVERS_UPC, $websiteScope,$websiteId);
            $quivers_preorder = $this->scopeConfig->getValue(self::XML_QUIVERS_PREORDER, $websiteScope, $websiteId);
            $debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $websiteScope, $websiteId);
            $quivers_pending_retailer_fulfillment_status = $this->scopeConfig->getValue(self::XML_QUIVERS_PENDING_RETAILER_FULFILLMENT_STATUS, $websiteScope, $websiteId);
            if($debug_mod){
                $log->debug_mode_on($debug_mod, "ConfigObserver.php execute function if condition is executing with websiteId-".$websiteId, null,null, 
                ["websiteId"=>$websiteId,
                "storeId"=> $storeId,
                "quiversSandbox" => $quiversSandbox,
                "active" => $active,
                "quivers_upc" => $quivers_upc,
                "quivers_preorder"=>$quivers_preorder,
                "quivers_pending_retailer_fulfillment_status"=>$quivers_pending_retailer_fulfillment_status,
                    ]
                );
            }
        }else{
            $marketPlaceId = $this->scopeConfig->getValue(self::XML_MARKETPLACE_ID, $storeScope,$storeId);
            $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope,$storeId);
            $eurMarketPlaceId = $this->scopeConfig->getValue(self::XML_EUR_MARKETPLACE_ID, $storeScope,$storeId);
            $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope,$storeId);
            $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope,$storeId);
            $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope,$storeId);
            $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope,$storeId);
            $productGroupId = $this->scopeConfig->getValue(self::XML_PRODUCTGROUP_ID, $storeScope,$storeId);
            $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope, $storeId);
            $magentoBaseUrl = $this->scopeConfig->getValue(self::XML_MAGENTO_BASE_URL, $storeScope,$storeId);
            $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope,$storeId);
            $quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $storeScope,$storeId);
            $magentoAcessToken = $this->scopeConfig->getValue(self::XML_MAGENTO_ACCESS_TOKEN, $storeScope,$storeId);
            $active = $this->scopeConfig->getValue(self::XML_ENABLED, $storeScope, $storeId);
            $quivers_upc = $this->scopeConfig->getValue(self::XML_QUIVERS_UPC, $storeScope, $storeId);
            $quivers_preorder = $this->scopeConfig->getValue(self::XML_QUIVERS_PREORDER, $storeScope, $storeId);
            $debug_mod = $this->scopeConfig->getValue('quivers/parameters/quivers_debug_mode', $storeScope, $storeId);
            $quivers_pending_retailer_fulfillment_status = $this->scopeConfig->getValue(self::XML_QUIVERS_PENDING_RETAILER_FULFILLMENT_STATUS, $websiteScope, $websiteId);
            if($debug_mod){
                $log->debug_mode_on($debug_mod, "ConfigObserver.php execute function if condition is executing with websiteId-".$websiteId, null,null, 
                ["websiteId"=>$websiteId,
                "storeId"=> $storeId,
                "quiversSandbox" => $quiversSandbox,
                "active" => $active,
                "quivers_upc" => $quivers_upc,
                "quivers_preorder"=>$quivers_preorder,
                "quivers_pending_retailer_fulfillment_status"=>$quivers_pending_retailer_fulfillment_status,
                    ]
                );
            }
        }  
        // $is_active = "false"; if($active){$is_active = "true";}
        if($quiversSandbox == 0) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }

        if (filter_var($magentoBaseUrl, FILTER_VALIDATE_URL) === FALSE) {
            $errorMessage="Invalid Magento Base Url. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.";
            $blockObj->getBackendSession()->unsMyValue();
            $customer_session->setSettingStatus("InActive");
               $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
            throw new CouldNotSaveException(
                __($errorMessage)); 
        }
        
        $this->validate_magento_creds($magentoBaseUrl,$magentoAcessToken,$blockObj,$quiversSandbox);
        $profileUrl = $this->baseUrl . "/create";
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";

        $product_group_id_array = explode(", ", $productGroupId);
        $product_group_id_list = [];
        foreach ($product_group_id_array as $productGroupId) {
            array_push($product_group_id_list, $productGroupId);
        }
        if($debug_mod){
        $log->debug_mode_on($debug_mod, "SaveQuiversSettings-".$quiversAPIKey, null,null, 
            ["websiteId"=>$websiteId,
            "storeId"=> $storeId,
            "quiversSandbox" => $quiversSandbox,
            "active" => $active,
            "quivers_upc" => $quivers_upc,
            "quivers_preorder"=>$quivers_preorder,
            "product_group_id_list"=>$product_group_id_list,
            "order_status"=>array(
                "quivers_pending_retailer_fulfillment_status"=>$quivers_pending_retailer_fulfillment_status
                )
            ]
            );
        }
        $payload = [
            "client_type" => "Magento2",
            "base_url" => $magentoBaseUrl,
            "business_refid" => $quiversBusinessRefId,
            "api_key" => $quiversAPIKey,
            "client_preorder_key" => $quivers_preorder,
            "client_upc_key" => $quivers_upc,
            "auth_token" => "Bearer ".$magentoAcessToken,
            "enable_plugin" => "true",
            "order_status"=>array(
                "quivers_pending_retailer_fulfillment_status"=>$quivers_pending_retailer_fulfillment_status
                )
        ];
        
        //Added store Ids to save in the profile (middleware)
        //These store Ids will be further used to fetch the orders from only those 
        //store for which Quivers plugin is active
        $store_ids = [];
        if ($storeId){
            if($active){
                $store_ids = [
                    "enable_store_ids" => $storeId
                ];
            }
            else{
                $store_ids = [
                    "disable_store_ids" => $storeId
                ];
            }
        }
        $payload = array_merge($payload, $store_ids);

        $this->curlClient->addHeader("Content-Type", "application/json");
        $this->curlClient->post($profileUrl, json_encode($payload));
        $id=[ 'api' => 'quivers_parameters_quivers_api_key',
                 'ref_id' => 'quivers_parameters_merchant_ref_id'];
        $response = $this->curlClient->getBody();
        if($debug_mod){
            $logData = [
                "Url" => $profileUrl,
                "Status" =>$this->curlClient->getStatus()
            ];
            $log->debug_mode_on($debug_mod,
                'Store Ids On Quivers ',
                $payload,  $response,$logData);
        }
        $blockObj->getBackendSession()->unsMyValue();
        $customer_session->setSettingStatus("InActive");
        $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
        $collection = $this->_productCollectionFactory->create();
        $collection->addAttributeToSelect('*');
        foreach ($collection as $product) 
        {   
            try
            {   
                if($product->getResource()->getAttribute('backend_upc'))
                {
                    $product->setData('backend_upc',$quivers_upc);
                    $product->getResource()->saveAttribute($product,'backend_upc');
                }
                else
                {
                    $message="This Product has no backend_upc field as attribute";
                    $log->print_statement($message);
                }
            }
            catch(\Exception $e)
            {
                $log->debug_mode_on($debug_mod,
			'Catch  "ConfigObserver.php -"Error Updating backend_upc field in product'.$quote->getBillingAddress()->getEmail(),
			null,  null,$e->getMessage());
                $message="Error Updating backend_upc field in product";
                $log->print_statement($message);

            }
        } 
        $message="Product update successfully";
        $log->print_statement($message);
        $customer_session->setSettingStatus("InActive");
        if($this->curlClient->getStatus()==200){
            $uuid = json_decode($response)->uuid;
            if($uuid) {
                $save = $this->configWriter->save(self::XML_QUIVERS_UUID, $uuid);
            }
            if (json_decode($response)->isactive=='true'){
                $blockObj->getBackendSession()->unsMyValue();
                $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "Active");
                $customer_session->setSettingStatus("Active");
            }
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $customer_session->setQuivers("true");
            // $websiteId = (int) $this->request->getParam('website',0);
            // $currentStoreId = $this->request->getParam('store', 0);
            // if($websiteId){
            //     $url = $this->_backendUrl->getUrl("adminhtml/system_config/edit/section/quivers_tax/website/".$websiteId);
            // }else{
            // $url = $this->_backendUrl->getUrl("adminhtml/system_config/edit/section/quivers_tax/store/".$currentStoreId);
            // }
            // $_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            // $this->_responseFactory->create()->setRedirect($url)->sendResponse();
            return;
        }
        $allMarketPlaces = [];

        $marketPlaceIds = [
            "USD" => $usdMarketPlaceId,
            "CAD" => $cadMarketPlaceId,
            "EUR" => $eurMarketPlaceId,
            "AUD" => $audMarketPlaceId,
            "JPY" => $jpyMarketPlaceId,
            "GBP" => $gbpMarketPlaceId
        ];
        foreach($marketPlaceIds as $currency=>$mPlaceId) {
            if($mPlaceId) {
                $allMarketPlaces[] = [
                    'marketplace_id' => $mPlaceId,
                    'currency' => $currency,
                    'claiming_group_ids' => $product_group_id_list         
                ];
            }
        }
        if($this->curlClient->getStatus()==200) {
            $response = json_decode($response, true);
            $uuid = $response["uuid"];
            if($debug_mod){
            $log->debug_mode_on($debug_mod,
			'Curl client responded with status OK'.$quote->getBillingAddress()->getEmail(),
			null,  $response,["Get Status"=>$this->curlClient->getStatus(),"uuid"=>$response["uuid"]]);    
            }          
            
            if($uuid) {
                $save = $this->configWriter->save(self::XML_QUIVERS_UUID, $uuid);
            }
            if(count($allMarketPlaces)>0) {
                $payload = [ "marketplaces" => $allMarketPlaces ];
                $this->curlClient->addHeader("uuid", $uuid);
                $this->curlClient->post($profileUpdateUrl, json_encode($payload));
                $response = $this->curlClient->getBody();
                if($this->curlClient->getStatus()>=400) {
                    $errorMessage = 'An error occurred while updating the profile. ERROR: '.$response;
                    throw new CouldNotSaveException(
                        __($errorMessage));
                }
            }
        } else {
            if($this->curlClient->getStatus()>=500){
                $errorMessage = json_decode($response)->message;
                $log->debug_mode_on($debug_mod,
			'Curl client responded with status >=500'.$quote->getBillingAddress()->getEmail(),
			null,  null,["Get Status"=>$this->curlClient->getStatus(),"Error Message"=>$errorMessage],);
                $customer_session->setSettingStatus("InActive");
            $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
            
            }
            else{
                $log->debug_mode_on($debug_mod,
                'Curl client responded with status < 500 and status!= "200"'.$quote->getBillingAddress()->getEmail(),
                null,  null,["Get Status"=>$this->curlClient->getStatus(),]);
            try{
                $customer_session->setSettingStatus("InActive");
            $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
            $errorMessage=json_decode($response)->error->message;
            $key=json_decode($response)->error->key;
            foreach($id as $keys=>$fieldId) {
            if($keys==$key) {
               $blockObj->getCatalogSession()->setMyName($fieldId); 
            }}}
            catch (\Exception $e){
                $log->debug_mode_on($debug_mod,
			'Catch  "ConfigObserver.php -"Invalid Form Details'.$quote->getBillingAddress()->getEmail(),
			null,  null,$e->getMessage());
                $customer_session->setSettingStatus("InActive");
                $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
                $errorMessage="Invalid Form Details Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.";
            }
            throw new CouldNotSaveException(
                __($errorMessage));
        }}

    }
    
    public function validate_magento_creds($magentoBaseUrl,$magentoAcessToken,$blockObj,$quiversSandbox){
        if ($quiversSandbox == 0 and parse_url($magentoBaseUrl)['scheme'] != 'https')
        {
            $errorMessage='Invalid Magento Base Url. Please ensure that if test mode is disabled , set the base URL to HTTPS. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
            $blockObj->getBackendSession()->unsMyValue();
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $customer_session=$objectManager->get('\Magento\Customer\Model\Session');
            $customer_session->setSettingStatus("InActive");
               $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
        throw new CouldNotSaveException(
                __($errorMessage));
        }
        // $endpoint = '/V1/orders';
        // $url = $magentoBaseUrl.$endpoint;
        // $date=date("Y-m-d H:i:s");
        // $params = array('searchCriteria[filter_groups][0][filters][0][field]'=>'status',
        //     'searchCriteria[filter_groups][0][filters][0][value]'=> 'pending',
        //     'searchCriteria[filter_groups][0][filters][0][condition_type]'=> 'eq',
        //     'searchCriteria[filter_groups][1][filters][0][field]'=> 'created_at',
        //     'searchCriteria[filter_groups][1][filters][0][value]'=> $date,
        //     'searchCriteria[filter_groups][1][filters][0][condition_type]'=> 'gteq',
        //     'searchCriteria[pageSize]'=> 1);
        // $url = $url.'?'.http_build_query($params);
        // $token='Bearer '.$magentoAcessToken;
        // $this->curlClient->addHeader("Authorization", $token);
        // try{
        // $this->curlClient->get($url);}
        // catch (\Exception $e) {
        //     $errorMessage='Invalid Magento Base Url or Basic Authorization token. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
        //     $blockObj->getBackendSession()->unsMyValue();
        //        $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
        //     throw new CouldNotSaveException(
        //         __($errorMessage));
        // }
        // $code=$this->curlClient->getStatus();
        // $response=$this->curlClient->getBody();
        // if($code>=400){
        // $blockObj->getBackendSession()->unsMyValue();
        //    $this->configWriter->save(self::XML_QUIVERS_STATUS_LABEL, "InActive");
        // if($code == 404){
        //     $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
        //     $errorMessage='Invalid Magento Base Url. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
        //     }
        // elseif($code == 401){
        //     $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_access_token');
        //     $errorMessage='Invalid Basic Authorization. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
        //     }
        // else {
        //     $errorMessage='Invalid Magento Base Url or Basic Authorization token. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';}
        // throw new CouldNotSaveException(
        //         __($errorMessage));
        //     }

    }
}
